==> cryptoexchanges-0.0.1.tgz: package/es/exchanges/base/BaseExchange.js <==
function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } } function _next(value) { step("next", value); } function _throw(err) { step("throw", err); } _next(); }); }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

import axios from 'axios';
import stringinject from 'stringinject';
import forEach from 'lodash/forEach';
import groupBy from 'lodash/groupBy';
import sortBy from 'lodash/sortBy';
import keyBy from 'lodash/keyBy';
import flatten from 'lodash/flatten';
import merge from 'lodash/merge';
import camelCase from 'lodash/camelCase';
import isString from 'lodash/isString';
import ExchangeError from './errors/ExchangeError';
import AuthenticationError from './errors/AuthenticationError';

var Exchange =
/*#__PURE__*/
function () {
  function Exchange() {
    var _this = this;

    var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
        apiKey = _ref.apiKey,
        apiSecret = _ref.apiSecret,
        uid = _ref.uid,
        password = _ref.password,
        _ref$includeInfo = _ref.includeInfo,
        includeInfo = _ref$includeInfo === void 0 ? false : _ref$includeInfo,
        _ref$verbose = _ref.verbose,
        verbose = _ref$verbose === void 0 ? false : _ref$verbose;

    _classCallCheck(this, Exchange);

    Object.defineProperty(this, "substituteCommonCurrencyCodes", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: true
    });
    Object.defineProperty(this, "adjustForTimeDifference", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: false
    });
    Object.defineProperty(this, "timeDifference", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: null
    });
    Object.defineProperty(this, "timeout", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: 1000
    });
    Object.defineProperty(this, "balance", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: {}
    });
    Object.defineProperty(this, "orderbooks", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: {}
    });
    Object.defineProperty(this, "tickers", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: {}
    });
    Object.defineProperty(this, "orders", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: {}
    });
    Object.defineProperty(this, "trades", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: {}
    });
    Object.defineProperty(this, "markets", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: null
    });
    Object.defineProperty(this, "marketsById", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: null
    });
    Object.defineProperty(this, "request", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: function value(method, baseUrl, path, requestConfig) {
        return _this.rawRequest(method, baseUrl, path, false, requestConfig);
      }
    });
    Object.defineProperty(this, "signedRequest", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: function value(method, baseUrl, path, requestConfig) {
        var containsAllParams = _this.constructor.REQUIRED_CREDENTIALS.every(function (param) {
          return _this[param];
        });

        if (!containsAllParams) {
          throw new AuthenticationError("Cannot sign request as ".concat(_this.constructor.REQUIRED_CREDENTIALS.join(', '), " required"));
        }

        return _this.rawRequest(method, baseUrl, path, true, requestConfig);
      }
    });
    Object.defineProperty(this, "setApiMethods", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: function value() {
        _this.api = {};
        forEach(_this.constructor.URLS.api, function (baseUrl, name) {
          _this.api[name] = {};
          forEach(_this.constructor.API[name], function (pathsArray, method) {
            _this.setApiMethodsFromArray(method, pathsArray, name, baseUrl);
          });
        });
      }
    });
    Object.defineProperty(this, "setApiMethodsFromArray", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: function value(method, pathsArray, name, baseUrl) {
        var isPrivate = _this.constructor.SIGNED_APIS.includes(name);

        pathsArray.forEach(function (path) {
          _this.api[name][method] = _this.api[name][method] || {};

          _this.api[name][method][camelCase(path)] = function (requestConfig) {
            var requestMethod = isPrivate ? _this.signedRequest : _this.request; // Handle paths that are `/order/{id}` to `/order/1`

            var injectedPath = requestConfig ? stringinject(path, requestConfig) : path;
            return requestMethod(method, baseUrl, injectedPath, requestConfig);
          };
        });
      }
    });
    this.apiKey = apiKey;
    this.apiSecret = apiSecret;
    this.uid = uid;
    this.password = password;
    this.verbose = verbose;
    this.validateRequiredConfig(apiKey, apiSecret, uid, password);
    this.validateParser();
    this.client = axios.create({
      timeout: this.timeout
    });
    this.parser = new this.constructor.Parser({
      exchange: this,
      includeInfo: includeInfo
    });
    this.setApiMethods();
  }

  _createClass(Exchange, [{
    key: "rawRequest",
    value: function () {
      var _rawRequest = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee(method, baseUrl, path) {
        var signed,
            requestConfig,
            options,
            response,
            _args = arguments;
        return regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                signed = _args.length > 3 && _args[3] !== undefined ? _args[3] : false;
                requestConfig = _args.length > 4 && _args[4] !== undefined ? _args[4] : {};
                options = _extends({
                  baseURL: baseUrl,
                  url: path,
                  headers: {},
                  method: method
                }, requestConfig);

                if (signed) {
                  merge(options, this.sign(_extends({}, options, requestConfig)));
                }

                _context.prev = 4;

                if (this.verbose) {
                  console.log("Request: ".concat(baseUrl, ", Path: ").concat(path, ", Options: ").concat(JSON.stringify(options), ", Config: ").concat(JSON.stringify(requestConfig)));
                }

                _context.next = 8;
                return this.client(options);

              case 8:
                response = _context.sent;
                return _context.abrupt("return", response.data);

              case 12:
                _context.prev = 12;
                _context.t0 = _context["catch"](4);

                // TODO: handle error
                if (_context.t0.response) {
                  // The request was made and the server responded with a status code
                  // that falls out of the range of 2xx
                  console.log(_context.t0.response.data);
                  console.log(_context.t0.response.status);
                  console.log(_context.t0.response.headers);
                } else if (_context.t0.request) {
                  // The request was made but no response was received
                  // `error.request` is an instance of XMLHttpRequest in the browser and an instance of
                  // http.ClientRequest in node.js
                  console.log(_context.t0.request);
                } else {
                  // Something happened in setting up the request that triggered an Error
                  console.log('Error', _context.t0.message);
                }

                console.log(_context.t0.config);
                return _context.abrupt("return", null);

              case 17:
              case "end":
                return _context.stop();
            }
          }
        }, _callee, this, [[4, 12]]);
      }));

      return function rawRequest(_x, _x2, _x3) {
        return _rawRequest.apply(this, arguments);
      };
    }()
  }, {
    key: "commonCurrencyCode",
    value: function commonCurrencyCode(currency) {
      if (!this.substituteCommonCurrencyCodes) return currency;
      if (currency === 'XBT') return 'BTC';
      if (currency === 'BCC') return 'BCH';
      if (currency === 'DRK') return 'DASH';
      return currency;
    }
  }, {
    key: "loadMarkets",
    value: function () {
      var _loadMarkets = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee2() {
        var reload,
            markets,
            currencies,
            _args2 = arguments;
        return regeneratorRuntime.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                reload = _args2.length > 0 && _args2[0] !== undefined ? _args2[0] : false;

                if (!(!reload && this.markets)) {
                  _context2.next = 5;
                  break;
                }

                if (this.marketsById) {
                  _context2.next = 4;
                  break;
                }

                return _context2.abrupt("return", this.setMarkets(this.markets));

              case 4:
                return _context2.abrupt("return", this.markets);

              case 5:
                _context2.next = 7;
                return this.fetchMarkets();

              case 7:
                markets = _context2.sent;

                if (!this.fetchCurrencies) {
                  _context2.next = 12;
                  break;
                }

                _context2.next = 11;
                return this.fetchCurrencies();

              case 11:
                currencies = _context2.sent;

              case 12:
                return _context2.abrupt("return", this.setMarkets(markets, currencies));

              case 13:
              case "end":
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));

      return function loadMarkets() {
        return _loadMarkets.apply(this, arguments);
      };
    }()
  }, {
    key: "setMarkets",
    value: function setMarkets(markets) {
      var _this2 = this;

      var currencies = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : undefined;
      var values = Object.values(markets).map(function (market) {
        return merge({
          limits: _this2.limits,
          precision: _this2.precision
        }, _this2.constructor.FEES.trading, market);
      });
      this.markets = merge(this.markets, keyBy(values, 'symbol'));
      this.marketsById = keyBy(markets, 'id');
      this.symbols = Object.keys(this.markets).sort();
      this.ids = Object.keys(this.marketsById).sort();

      if (currencies) {
        this.currencies = merge(currencies, this.currencies);
      } else {
        var baseCurrencies = values.filter(function (market) {
          return 'base' in market;
        }).map(function (market) {
          return {
            id: market.baseId || market.base,
            code: market.base,
            precision: market.precision ? market.precision.base || market.precision.amount : 8
          };
        });
        var quoteCurrencies = values.filter(function (market) {
          return 'quote' in market;
        }).map(function (market) {
          return {
            id: market.quoteId || market.quote,
            code: market.quote,
            precision: market.precision ? market.precision.quote || market.precision.price : 8
          };
        });
        var allCurrencies = baseCurrencies.concat(quoteCurrencies);
        var groupedCurrencies = groupBy(allCurrencies, 'code');
        var currentCurrencies = Object.keys(groupedCurrencies).map(function (code) {
          return groupedCurrencies[code].reduce(function (previous, current) {
            return previous.precision > current.precision ? previous : current;
          }, groupedCurrencies[code][0]);
        });
        var sortedCurrencies = sortBy(flatten(currentCurrencies), 'code');
        this.currencies = merge(keyBy(sortedCurrencies, 'code'), this.currencies);
      }

      this.currencies_by_id = keyBy(this.currencies, 'id');
      return this.markets;
    }
  }, {
    key: "marketId",
    value: function marketId(symbol) {
      return this.market(symbol).id || symbol;
    }
  }, {
    key: "market",
    value: function market(symbol) {
      if (!this.markets) {
        return new ExchangeError("".concat(this.constructor.name, " markets not loaded"));
      }

      if (!symbol) {
        throw new ExchangeError('Symbol passed to market() is undefined or null');
      }

      if (isString(symbol) && this.markets[symbol]) {
        return this.markets[symbol];
      }

      throw new ExchangeError("".concat(this.constructor.name, " does not have market symbol ").concat(symbol));
    }
  }, {
    key: "costToPrecision",
    value: function costToPrecision(symbol, cost) {
      return parseFloat(cost).toFixed(this.markets[symbol].precision.price);
    }
  }, {
    key: "feeToPrecision",
    value: function feeToPrecision(symbol, fee) {
      return parseFloat(fee).toFixed(this.markets[symbol].precision.price);
    }
  }, {
    key: "amountToPrecision",
    value: function amountToPrecision(symbol, amount) {
      return this.truncate(amount, this.markets[symbol].precision.amount);
    }
  }, {
    key: "amountToString",
    value: function amountToString(symbol, amount) {
      return this.truncateToString(amount, this.markets[symbol].precision.amount);
    }
  }, {
    key: "priceToPrecision",
    value: function priceToPrecision(symbol, price) {
      return parseFloat(price).toFixed(this.markets[symbol].precision.price);
    } // TODO: figure this out
    // eslint-disable-next-line

  }, {
    key: "truncateToString",
    value: function truncateToString(number, precision) {
      return number && number.toFixed(precision);
    } // Validations
    // TODO: Move this elsewhere

  }, {
    key: "validateRequiredConfig",
    value: function validateRequiredConfig(apiKey, apiSecret, uid, password) {
      var _this3 = this;

      if (apiKey || apiSecret || uid || password) {
        var containsAllParams = this.constructor.REQUIRED_CREDENTIALS.every(function (param) {
          return _this3[param];
        });

        if (!containsAllParams) {
          throw new ExchangeError('Does not have all required params');
        }
      }
    }
  }, {
    key: "validateParser",
    value: function validateParser() {
      if (!this.constructor.Parser) {
        throw new ExchangeError('Does not have parser set');
      }
    } // Methods to override
    // eslint-disable-next-line class-methods-use-this

  }, {
    key: "getHeaders",
    value: function getHeaders() {
      return {};
    }
  }]);

  return Exchange;
}();

export default Exchange;

==> cryptoexchanges-0.0.2.tgz: package/es/exchanges/base/BaseExchange.js <==
function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } } function _next(value) { step("next", value); } function _throw(err) { step("throw", err); } _next(); }); }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

import axios from 'axios';
import forEach from 'lodash/forEach';
import groupBy from 'lodash/groupBy';
import sortBy from 'lodash/sortBy';
import keyBy from 'lodash/keyBy';
import flatten from 'lodash/flatten';
import merge from 'lodash/merge';
import camelCase from 'lodash/camelCase';
import isString from 'lodash/isString';
import ExchangeError from './errors/ExchangeError';
import AuthenticationError from './errors/AuthenticationError';
import stringinject from '../../utils/stringinject';
import debug from '../../utils/debug';

var Exchange =
/*#__PURE__*/
function () {
  function Exchange() {
    var _this = this;

    var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
        apiKey = _ref.apiKey,
        apiSecret = _ref.apiSecret,
        uid = _ref.uid,
        password = _ref.password,
        _ref$includeInfo = _ref.includeInfo,
        includeInfo = _ref$includeInfo === void 0 ? false : _ref$includeInfo,
        _ref$verbose = _ref.verbose,
        verbose = _ref$verbose === void 0 ? false : _ref$verbose;

    _classCallCheck(this, Exchange);

    Object.defineProperty(this, "substituteCommonCurrencyCodes", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: true
    });
    Object.defineProperty(this, "adjustForTimeDifference", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: false
    });
    Object.defineProperty(this, "timeDifference", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: null
    });
    Object.defineProperty(this, "timeout", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: 1000
    });
    Object.defineProperty(this, "balance", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: {}
    });
    Object.defineProperty(this, "orderbooks", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: {}
    });
    Object.defineProperty(this, "tickers", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: {}
    });
    Object.defineProperty(this, "orders", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: {}
    });
    Object.defineProperty(this, "trades", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: {}
    });
    Object.defineProperty(this, "markets", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: null
    });
    Object.defineProperty(this, "marketsById", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: null
    });
    Object.defineProperty(this, "request", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: function value(method, baseUrl, path, requestConfig) {
        return _this.rawRequest(method, baseUrl, path, false, requestConfig);
      }
    });
    Object.defineProperty(this, "signedRequest", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: function value(method, baseUrl, path, requestConfig) {
        var containsAllParams = _this.constructor.REQUIRED_CREDENTIALS.every(function (param) {
          return _this[param];
        });

        if (!containsAllParams) {
          throw new AuthenticationError("Cannot sign request as ".concat(_this.constructor.REQUIRED_CREDENTIALS.join(', '), " required"));
        }

        return _this.rawRequest(method, baseUrl, path, true, requestConfig);
      }
    });
    Object.defineProperty(this, "setApiMethods", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: function value() {
        _this.api = {};
        forEach(_this.constructor.URLS.api, function (baseUrl, name) {
          _this.api[name] = {};
          forEach(_this.constructor.API[name], function (pathsArray, method) {
            _this.setApiMethodsFromArray(method, pathsArray, name, baseUrl);
          });
        });
      }
    });
    Object.defineProperty(this, "setApiMethodsFromArray", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: function value(method, pathsArray, name, baseUrl) {
        var isPrivate = _this.constructor.SIGNED_APIS.includes(name);

        pathsArray.forEach(function (path) {
          _this.api[name][method] = _this.api[name][method] || {};

          _this.api[name][method][camelCase(path)] = function (requestConfig) {
            var requestMethod = isPrivate ? _this.signedRequest : _this.request; // Handle paths that are `/order/{id}` to `/order/1`

            var injectedPath = requestConfig ? stringinject(path, requestConfig) : path;
            return requestMethod(method, baseUrl, injectedPath, requestConfig);
          };
        });
      }
    });
    this.apiKey = apiKey;
    this.apiSecret = apiSecret;
    this.uid = uid;
    this.password = password;
    this.verbose = verbose;
    this.validateRequiredConfig(apiKey, apiSecret, uid, password);
    this.validateParser();
    this.client = axios.create({
      timeout: this.timeout
    });
    this.parser = new this.constructor.Parser({
      exchange: this,
      includeInfo: includeInfo
    });
    this.setApiMethods();
  }

  _createClass(Exchange, [{
    key: "rawRequest",
    value: function () {
      var _rawRequest = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee(method, baseUrl, path) {
        var signed,
            requestConfig,
            options,
            response,
            _args = arguments;
        return regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                signed = _args.length > 3 && _args[3] !== undefined ? _args[3] : false;
                requestConfig = _args.length > 4 && _args[4] !== undefined ? _args[4] : {};
                options = _extends({
                  baseURL: baseUrl,
                  url: path,
                  headers: {},
                  method: method
                }, requestConfig);

                if (signed) {
                  merge(options, this.sign(_extends({}, options, requestConfig)));
                }

                _context.prev = 4;

                if (this.verbose) {
                  debug("Request: ".concat(baseUrl, ", Path: ").concat(path, ", Options: ").concat(JSON.stringify(options), ", Config: ").concat(JSON.stringify(requestConfig)));
                }

                _context.next = 8;
                return this.client(options);

              case 8:
                response = _context.sent;
                return _context.abrupt("return", response.data);

              case 12:
                _context.prev = 12;
                _context.t0 = _context["catch"](4);

                // TODO: handle error
                if (_context.t0.response) {
                  // The request was made and the server responded with a status code
                  // that falls out of the range of 2xx
                  console.log(_context.t0.response.data);
                  console.log(_context.t0.response.status);
                  console.log(_context.t0.response.headers);
                } else if (_context.t0.request) {
                  // The request was made but no response was received
                  // `error.request` is an instance of XMLHttpRequest in the browser and an instance of
                  // http.ClientRequest in node.js
                  console.log(_context.t0.request);
                } else {
                  // Something happened in setting up the request that triggered an Error
                  console.log('Error', _context.t0.message);
                }

                console.log(_context.t0.config);
                throw _context.t0;

              case 17:
              case "end":
                return _context.stop();
            }
          }
        }, _callee, this, [[4, 12]]);
      }));

      return function rawRequest(_x, _x2, _x3) {
        return _rawRequest.apply(this, arguments);
      };
    }()
  }, {
    key: "commonCurrencyCode",
    value: function commonCurrencyCode(currency) {
      if (!this.substituteCommonCurrencyCodes) return currency;
      if (currency === 'XBT') return 'BTC';
      if (currency === 'BCC') return 'BCH';
      if (currency === 'DRK') return 'DASH';
      return currency;
    }
  }, {
    key: "loadMarkets",
    value: function () {
      var _loadMarkets = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee2() {
        var reload,
            markets,
            currencies,
            _args2 = arguments;
        return regeneratorRuntime.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                reload = _args2.length > 0 && _args2[0] !== undefined ? _args2[0] : false;

                if (!(!reload && this.markets)) {
                  _context2.next = 5;
                  break;
                }

                if (this.marketsById) {
                  _context2.next = 4;
                  break;
                }

                return _context2.abrupt("return", this.setMarkets(this.markets));

              case 4:
                return _context2.abrupt("return", this.markets);

              case 5:
                _context2.next = 7;
                return this.fetchMarkets();

              case 7:
                markets = _context2.sent;

                if (!this.fetchCurrencies) {
                  _context2.next = 12;
                  break;
                }

                _context2.next = 11;
                return this.fetchCurrencies();

              case 11:
                currencies = _context2.sent;

              case 12:
                return _context2.abrupt("return", this.setMarkets(markets, currencies));

              case 13:
              case "end":
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));

      return function loadMarkets() {
        return _loadMarkets.apply(this, arguments);
      };
    }()
  }, {
    key: "setMarkets",
    value: function setMarkets(markets) {
      var _this2 = this;

      var currencies = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : undefined;
      var values = Object.values(markets).map(function (market) {
        return merge({
          limits: _this2.limits,
          precision: _this2.precision
        }, _this2.constructor.FEES.trading, market);
      });
      this.markets = merge(this.markets, keyBy(values, 'symbol'));
      this.marketsById = keyBy(markets, 'id');
      this.symbols = Object.keys(this.markets).sort();
      this.ids = Object.keys(this.marketsById).sort();

      if (currencies) {
        this.currencies = merge(currencies, this.currencies);
      } else {
        var baseCurrencies = values.filter(function (market) {
          return 'base' in market;
        }).map(function (market) {
          return {
            id: market.baseId || market.base,
            code: market.base,
            precision: market.precision ? market.precision.base || market.precision.amount : 8
          };
        });
        var quoteCurrencies = values.filter(function (market) {
          return 'quote' in market;
        }).map(function (market) {
          return {
            id: market.quoteId || market.quote,
            code: market.quote,
            precision: market.precision ? market.precision.quote || market.precision.price : 8
          };
        });
        var allCurrencies = baseCurrencies.concat(quoteCurrencies);
        var groupedCurrencies = groupBy(allCurrencies, 'code');
        var currentCurrencies = Object.keys(groupedCurrencies).map(function (code) {
          return groupedCurrencies[code].reduce(function (previous, current) {
            return previous.precision > current.precision ? previous : current;
          }, groupedCurrencies[code][0]);
        });
        var sortedCurrencies = sortBy(flatten(currentCurrencies), 'code');
        this.currencies = merge(keyBy(sortedCurrencies, 'code'), this.currencies);
      }

      this.currencies_by_id = keyBy(this.currencies, 'id');
      return this.markets;
    }
  }, {
    key: "marketId",
    value: function marketId(symbol) {
      return this.market(symbol).id || symbol;
    }
  }, {
    key: "market",
    value: function market(symbol) {
      if (!this.markets) {
        return new ExchangeError("".concat(this.constructor.name, " markets not loaded"));
      }

      if (!symbol) {
        throw new ExchangeError('Symbol passed to market() is undefined or null');
      }

      if (isString(symbol) && this.markets[symbol]) {
        return this.markets[symbol];
      }

      throw new ExchangeError("".concat(this.constructor.name, " does not have market symbol ").concat(symbol));
    }
  }, {
    key: "costToPrecision",
    value: function costToPrecision(symbol, cost) {
      return parseFloat(cost).toFixed(this.markets[symbol].precision.price);
    }
  }, {
    key: "feeToPrecision",
    value: function feeToPrecision(symbol, fee) {
      return parseFloat(fee).toFixed(this.markets[symbol].precision.price);
    }
  }, {
    key: "amountToPrecision",
    value: function amountToPrecision(symbol, amount) {
      return this.truncate(amount, this.markets[symbol].precision.amount);
    }
  }, {
    key: "amountToString",
    value: function amountToString(symbol, amount) {
      return this.truncateToString(amount, this.markets[symbol].precision.amount);
    }
  }, {
    key: "priceToPrecision",
    value: function priceToPrecision(symbol, price) {
      return parseFloat(price).toFixed(this.markets[symbol].precision.price);
    } // TODO: figure this out
    // eslint-disable-next-line

  }, {
    key: "truncateToString",
    value: function truncateToString(number, precision) {
      return number && number.toFixed(precision);
    } // Validations
    // TODO: Move this elsewhere

  }, {
    key: "validateRequiredConfig",
    value: function validateRequiredConfig(apiKey, apiSecret, uid, password) {
      var _this3 = this;

      if (apiKey || apiSecret || uid || password) {
        var containsAllParams = this.constructor.REQUIRED_CREDENTIALS.every(function (param) {
          return _this3[param];
        });

        if (!containsAllParams) {
          throw new ExchangeError('Does not have all required params');
        }
      }
    }
  }, {
    key: "validateParser",
    value: function validateParser() {
      if (!this.constructor.Parser) {
        throw new ExchangeError('Does not have parser set');
      }
    } // Methods to override
    // eslint-disable-next-line class-methods-use-this

  }, {
    key: "getHeaders",
    value: function getHeaders() {
      return {};
    }
  }]);

  return Exchange;
}();

export default Exchange;
